#!/bin/bash
#
# Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

set -o errexit
set -o nounset
set -o pipefail
set -o errtrace

# variables
REGISTRY=container-registry.oracle.com/kubernetes
K8S_VERSION=v1.12.10-3
DNS_VERSION=1.14.13
FLANNEL_VERSION=v0.10.0-1
DASHBOARD_VERSION=v1.10.1-1
ETCD_VERSION=3.2.24-1
PAUSE_VERSION=3.1
COREDNS_VERSION=1.3.1
ARCH=""

function ops {
  local ops="${1:-}"
  local registry="${2:-${REGISTRY}}"

  docker ${ops} ${registry}/flannel:${FLANNEL_VERSION}${ARCH}
  docker ${ops} ${registry}/etcd${ARCH}:${ETCD_VERSION}
  docker ${ops} ${registry}/pause${ARCH}:${PAUSE_VERSION}
  docker ${ops} ${registry}/kube-controller-manager${ARCH}:${K8S_VERSION}
  docker ${ops} ${registry}/kube-scheduler${ARCH}:${K8S_VERSION}
  docker ${ops} ${registry}/kube-apiserver${ARCH}:${K8S_VERSION}
  docker ${ops} ${registry}/kube-proxy${ARCH}:${K8S_VERSION}
  docker ${ops} ${registry}/k8s-dns-sidecar${ARCH}:${DNS_VERSION}
  docker ${ops} ${registry}/k8s-dns-kube-dns${ARCH}:${DNS_VERSION}
  docker ${ops} ${registry}/k8s-dns-dnsmasq-nanny${ARCH}:${DNS_VERSION}
  docker ${ops} ${registry}/kubernetes-dashboard:${DASHBOARD_VERSION}
  if [ ! -z "${COREDNS_VERSION}" ]; then
  	docker ${ops} ${registry}/coredns:${COREDNS_VERSION} || return 0
  fi
}

function tag {
  local from="${1:-${REGISTRY}}"
  local to="${2:-}"

  docker tag ${from}/flannel:${FLANNEL_VERSION}${ARCH} ${to}/flannel:${FLANNEL_VERSION}${ARCH}
  docker tag ${from}/etcd${ARCH}:${ETCD_VERSION} ${to}/etcd${ARCH}:${ETCD_VERSION}
  docker tag ${from}/pause${ARCH}:${PAUSE_VERSION} ${to}/pause${ARCH}:${PAUSE_VERSION}
  docker tag ${from}/kube-controller-manager${ARCH}:${K8S_VERSION} ${to}/kube-controller-manager${ARCH}:${K8S_VERSION}
  docker tag ${from}/kube-scheduler${ARCH}:${K8S_VERSION} ${to}/kube-scheduler${ARCH}:${K8S_VERSION}
  docker tag ${from}/kube-apiserver${ARCH}:${K8S_VERSION} ${to}/kube-apiserver${ARCH}:${K8S_VERSION}
  docker tag ${from}/kube-proxy${ARCH}:${K8S_VERSION} ${to}/kube-proxy${ARCH}:${K8S_VERSION}
  docker tag ${from}/k8s-dns-sidecar${ARCH}:${DNS_VERSION} ${to}/k8s-dns-sidecar${ARCH}:${DNS_VERSION}
  docker tag ${from}/k8s-dns-kube-dns${ARCH}:${DNS_VERSION} ${to}/k8s-dns-kube-dns${ARCH}:${DNS_VERSION}
  docker tag ${from}/k8s-dns-dnsmasq-nanny${ARCH}:${DNS_VERSION} ${to}/k8s-dns-dnsmasq-nanny${ARCH}:${DNS_VERSION}
  docker tag ${from}/kubernetes-dashboard:${DASHBOARD_VERSION} ${to}/kubernetes-dashboard:${DASHBOARD_VERSION}
  if [ ! -z "${COREDNS_VERSION}" ]; then
        docker tag ${from}/coredns:${COREDNS_VERSION} ${to}/coredns:${COREDNS_VERSION} || return 0
  fi
}

function check {
  local from="${1:-${REGISTRY}}"
  local to="${2:-}"
  local version="${3:-}"
  local image="pause"

  # start docker-engine
  echo "Checking if docker is active ..."
  if ! systemctl -q is-active docker; then
        echo "[ERROR] docker is not started ... please start docker"
        exit 1
  fi

  version=$(echo ${version} | sed 's/v//g')
  if [ "${version}" = "1.7.4" ]; then
        K8S_VERSION=v${version}
        DNS_VERSION=1.14.4
        FLANNEL_VERSION=v0.7.1
        ETCD_VERSION=3.0.17
        PAUSE_VERSION=3.0
        DASHBOARD_VERSION=v1.7.0
  elif [ "${version}" = "1.8.4" ]; then
        K8S_VERSION=v${version}
        DNS_VERSION=1.14.5
        FLANNEL_VERSION=v0.9.0
        ETCD_VERSION=3.0.17
        PAUSE_VERSION=3.0
        DASHBOARD_VERSION=v1.7.0
   elif [ "${version}" = "1.9.1" ] || [ "${version}" = "1.9.1-1" ] || [ "${version}" = "1.9.11" ]; then
        K8S_VERSION=v${version}
        DNS_VERSION=1.14.7
        FLANNEL_VERSION=v0.9.0
	if [ "${version}" = "1.9.11" ]; then
		ETCD_VERSION=3.1.11
	else
		ETCD_VERSION=3.1.10
	fi
        PAUSE_VERSION=3.0
        DASHBOARD_VERSION=v1.7.0
  elif [ "${version}" = "1.10.5" ]; then
	K8S_VERSION=v${version}
	DNS_VERSION=1.14.8
	FLANNEL_VERSION=v0.9.0
	DASHBOARD_VERSION=v1.7.0
	ETCD_VERSION=3.1.12
	PAUSE_VERSION=3.1
  elif [ "${version}" = "1.11.3" ]; then
        K8S_VERSION=v${version}
        DNS_VERSION=1.14.10
        FLANNEL_VERSION=v0.9.0
        DASHBOARD_VERSION=v1.7.0
        ETCD_VERSION=3.2.18
        PAUSE_VERSION=3.1
  elif  [ "v${version}" = "${K8S_VERSION}" ]; then
	:
  else
        echo "[ERROR] ${version} is not a valid version"
        exit 1
  fi

  min=$(echo ${version}| awk -F. '{print $2}')
  if [ "${min}" -lt "12" ]; then
        ARCH="-amd64"
  fi

  echo "Checking for pull access from ${from} registry ..."
  local result=$(echo ""|docker pull ${from}/${image}${ARCH}:${PAUSE_VERSION} 2>&1|grep "Please login")
  if [ -n "${result}" ]; then
        echo "[ERROR] Please login with valid credential to the ${from}"
        echo "        # docker login ${from}"
        exit 1
  else
        if ! docker pull ${from}/${image}${ARCH}:${PAUSE_VERSION}; then
                echo "[ERROR] docker cannot pull ${image}:${PAUSE_VERSION} from ${from} registry"
                exit 1
        fi
  fi

  docker tag ${from}/${image}${ARCH}:${PAUSE_VERSION} ${to}/${image}${ARCH}:${PAUSE_VERSION}

  echo "Checking for push access to ${to} registry ..."
  if ! docker push ${to}/${image}${ARCH}:${PAUSE_VERSION}; then
	echo "[ERROR] docker cannot push ${image}${ARCH}:${PAUSE_VERSION} to ${to} registry"
	exit 1
  fi
}

function usage {
	echo "This script is to help pulling docker images from default container-registry.oracle.com to a local repo" >&2
	echo "usage: " >&2
	echo "  $0 --to registry [--from registry --version ${K8S_VERSION}]" >&2
	exit 1
}

# MAIN
ARG=($@)
PASS="0"
VERSION="${K8S_VERSION}"
FROM="${REGISTRY}"
for((i=0; i<$#; i++)); do
	if [ "${ARG[${i}]}" = "--to" ]; then
		PASS="1"
		i=$((i+1))
		TO="${ARG[${i}]:-}"
		if [ -z "${TO}" ]; then
			echo "[ERROR] Please provide a valid local registry location"
			exit 1
		fi
		# remove trailing / in the repository string
		TO=$(echo ${TO} |  sed 's/\/*$//g')
	elif [ "${ARG[${i}]}" = "--from" ]; then
		i=$((i+1))
		FROM="${ARG[${i}]:-${REGISTRY}}"
		# remove trailing / in the repository string
		FROM=$(echo ${FROM} |  sed 's/\/*$//g')
	elif [ "${ARG[${i}]}" = "--version" ]; then
		i=$((i+1))
		VERSION="${ARG[${i}]:-"${K8S_VERSION}"}"
	else
		usage
	fi
done

if [ "${PASS}" != "1" ]; then
	usage
fi

if [ -z "${LOCAL+x}" ]; then
	echo "Pulling images [${VERSION}] from: [${FROM}] to: [${TO}]"
	check "${FROM}" "${TO}" "${VERSION}"
	ops "pull" "${FROM}" # pull from registry
else
	VERSION="${K8S_VERSION}"
	ls /usr/local/share/kubeadm/*.tar | xargs -n1 docker load -i
fi

if [ "${VERSION}" != "${K8S_VERSION}" ]; then
	COREDNS_VERSION=""
fi

tag "${FROM}" "${TO}"
ops "push" "${TO}" # push to registry
echo "[SUCCESS] All images pushed to [${TO}]"
